from abc import ABC

"""
   Job Base class
"""
from typing import (
    Any,
    Generic
)

from starfish.types import TJobBase


class JobBase(ABC, Generic[TJobBase]):
    """
        Create a Job object

        :param agent: agent object that created the listing.
        :type agent: :class:`.Agent` object to assign to this Listing
        :param str job_id: id of the job.
    """

    IsWorkingStatusList = [
        'scheduled',
        'running',
        'accepted'
    ]

    def __init__(self, job_id: str, status: str = None, outputs: Any = None) -> None:
        """

        init the the Job Object Base with the agent instance

        :param int job_id: id of the job
        :param str status: status of the job
        :param dict outputs: dict or None for the outputs
        """
        self._job_id = job_id
        self._status = status
        self._outputs = outputs
        super().__init__()

    @property
    def is_finished(self) -> bool:
        if self._status:
            return self._status not in JobBase.IsWorkingStatusList
        return False

    @property
    def job_id(self) -> str:
        """
        Return the job id

        :return: Job id
        :type: int
        """
        return self._job_id

    @property
    def status(self) -> str:
        """
        Return the status for this job

        :return: status
        :type: str
        """
        return self._status

    @property
    def outputs(self) -> Any:
        """
        Return the ouptuts for this job

        :return: results
        :type: dict or None
        """
        return self._outputs

    def __str__(self) -> str:
        text = 'job:'
        if self._status:
            text += f'{self._status}'
        if self._outputs:
            text += f' = {self._outputs}'
        return text



"""
    starfish-ddo-registry contract

"""

from starfish.network.convex.contract.contract_base import ContractBase
from starfish.network.convex.convex_account import ConvexAccount
from starfish.network.convex.convex_network import ConvexNetwork
from starfish.types import AccountAddress


class ProvenanceContract(ContractBase):

    def __init__(self, convex: ConvexNetwork):
        ContractBase.__init__(self, convex, 'starfish-provenance')

    def register(self, asset_id: str, account: ConvexAccount):
        command = f'(call {self.address} (register {asset_id}))'
        result = self._convex.send(command, account)
        if result and 'value' in result:
            return result['value']
        return result

    def event_list(self, asset_id: str, account_address: AccountAddress):
        command = f'(call {self.address} (event-list {asset_id}))'
        if isinstance(account_address, str):
            address = account_address
        else:
            address = account_address.address
        result = self._convex.query(command, address)
        if result and 'value' in result:
            return result['value']
        return result

    def event_owner(self, account_address: AccountAddress):
        if isinstance(account_address, str):
            address = account_address
        else:
            address = account_address.address
        command = f'(call {self.address} (event-owner {address}))'
        result = self._convex.query(command, address)
        if result and 'value' in result:
            return result['value']
        return result



"""
    starfish-ddo-registry contract

"""
import re

from starfish.network.convex.contract.contract_base import ContractBase
from starfish.network.convex.convex_account import ConvexAccount
from starfish.network.convex.convex_network import ConvexNetwork
from starfish.types import AccountAddress


class DDORegistryContract(ContractBase):

    def __init__(self, convex: ConvexNetwork):
        ContractBase.__init__(self, convex, 'starfish-ddo-registry')

    def register_did(self, did: str, ddo_text: str, account: ConvexAccount):
        encode_ddo_text = re.sub('"', '\\"', ddo_text)
        command = f'(call {self.address} (register {did} "{encode_ddo_text}"))'
        result = self._convex.send(command, account)
        if result and 'value' in result:
            return result['value']
        return result

    def resolve(self, did: str, account_address: AccountAddress):
        command = f'(call {self.address} (resolve {did}))'
        if isinstance(account_address, str):
            address = account_address
        else:
            address = account_address.address
        result = self._convex.query(command, address)
        if result and 'value' in result:
            return result['value']
        return result

    def owner(self, did: str,  account_address: AccountAddress):
        command = f'(call {self.address} (owner {did}))'
        if isinstance(account_address, str):
            address = account_address
        else:
            address = account_address.address
        result = self._convex.query(command, address)
        if result and 'value' in result:
            return result['value']
        return result
